package turtle;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.Document;
import javax.swing.text.BadLocationException;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Reader;
import java.lang.reflect.InvocationTargetException;

import turtle.talk.Facade;
import turtle.talk.StringSink;
import turtle.talk.IntSink;
import turtle.talk.IntSource;
import turtle.talk.Assert;

/** The main window for the Turtle-talk application.
 * <p>Title: Turtle-talk</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Memorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 */

class MainFrame extends JFrame {
    JPanel contentPane;
    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu jMenuFile = new JMenu();
    JMenuItem jMenuFileExit = new JMenuItem();
    JMenu jMenuHelp = new JMenu();
    JMenuItem jMenuHelpAbout = new JMenuItem();
    JLabel statusBar = new JLabel();
    JSplitPane jSplitPane1 = new JSplitPane();
    BorderLayout borderLayout1 = new BorderLayout();
    Facade facade = new Facade() ;
    JPanel jPanel1 = new MazePanel( facade.getMaze() );
    JMenu runMenu = new JMenu();
    JMenuItem menuItemCompile = new JMenuItem();
    JMenuItem menuItemRun = new JMenuItem();
    JSplitPane jSplitPane2 = new JSplitPane();
    JScrollPane jScrollPane1 = new JScrollPane();
    JScrollPane jScrollPane2 = new JScrollPane();
    JTextPane editTextComponent = new JTextPane();
    JTextPane messageTextComponent = new JTextPane();
    StringSink stringSink = new StringSink() {
        public void putString( String str ) {
            Document doc = messageTextComponent.getDocument() ;
            try{
                doc.insertString(doc.getLength(), str, null);
                messageTextComponent.setCaretPosition( doc.getLength() ); }
            catch( BadLocationException e) {
                Assert.assertTrue( false, "Unreachable reached" ) ; } }
        public void close() {} 
    } ;
    IntSink intSink = new IntSink() {
        public void putInt( int i ) {
            Document doc = messageTextComponent.getDocument() ;
            try{
                doc.insertString(doc.getLength(), ""+i, null); }
            catch( BadLocationException e) {
                Assert.assertTrue( false, "Unreachable reached" ) ; } }
        public void close() {} 
    } ;
    BoundedRangeModel inputInt = new DefaultBoundedRangeModel(0,0,Integer.MIN_VALUE,Integer.MAX_VALUE ) ;
    GetIntDialog getIntDialog = new GetIntDialog( this, inputInt ) ;
    IntSource intSource = new IntSource() {
    	// @TODO. If the program keeps asking for ints, in an infinite loop,
    	// there is no way to stop the program. It just keeps popping
    	// up a dialog and thus annoying the crap out of the user.
    	// More generally we need a way to stop infinite loops.
        public int getInt() {
        	class DoRun implements Runnable {
				public void run() {
		            getIntDialog.show() ;
				}
        		
        	}
        	try {
				SwingUtilities.invokeAndWait(new DoRun()) ;
			} catch (InterruptedException e) {
				Assert.assertTrue( false, "Unreachable" ) ;
			} catch (InvocationTargetException e) {
				Assert.assertTrue( false, "Unreachable" ) ;
			}
            return inputInt.getValue() ;
        }
        public void close() {}
    } ;
    JCheckBoxMenuItem traceMenuItem = new JCheckBoxMenuItem();
    JRadioButtonMenuItem slowMenuItem = new JRadioButtonMenuItem();
    JRadioButtonMenuItem mediumMenuItem = new JRadioButtonMenuItem();
    JRadioButtonMenuItem fastMenuItem = new JRadioButtonMenuItem();
    JMenuItem jMenuFileLoad = new JMenuItem();
    JMenuItem jMenuFileSave = new JMenuItem();
    JFileChooser fileChooser = new JFileChooser(".") ;
    JMenuItem randomizeMenuItem = new JMenuItem();

    //Construct the frame
    public MainFrame() {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
            jbInit();
            setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE ) ;
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
    //Component initialization
    private void jbInit() throws Exception  {
        //setIconImage(Toolkit.getDefaultToolkit().createImage(MainFrame.class.getResource("[Your Icon]")));
        contentPane = (JPanel) this.getContentPane();
        contentPane.setLayout(borderLayout1);
        this.setSize(new Dimension(400, 300));
        this.setTitle("Turtle Talk");
        statusBar.setText(" ");
        jMenuFile.setText("File");
        jMenuFileExit.setText("Exit");
        jMenuFileExit.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                jMenuFileExit_actionPerformed(e);
            }
        });
        jMenuHelp.setText("Help");
        jMenuHelpAbout.setText("About");
        jMenuHelpAbout.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                jMenuHelpAbout_actionPerformed(e);
            }
        });
        runMenu.setText("Run") ;
        menuItemCompile.setText("Compile");
        menuItemRun.setText("Run");
        jPanel1.setMinimumSize(new Dimension(200, 200));
        jPanel1.setPreferredSize(new Dimension(200, 200));
        jSplitPane2.setOrientation(JSplitPane.VERTICAL_SPLIT);
        jSplitPane2.setResizeWeight(0.5);
        messageTextComponent.setBackground(Color.lightGray);
        traceMenuItem.setText("Trace");
        ButtonGroup group = new ButtonGroup() ;
        jSplitPane1.setResizeWeight(0.5);
        contentPane.setMinimumSize(new Dimension(600, 800));
        jMenuFileLoad.setText("Load");
        jMenuFileLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                loadFile(e);
            }
        });
        jMenuFileSave.setText("Save");
        jMenuFileSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveFile(e);
            }
        });
        randomizeMenuItem.setText("Randomize");
        randomizeMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                randomize(e);
            }
        });
        group.add( slowMenuItem );
        group.add( mediumMenuItem );
        group.add( fastMenuItem );
        mediumMenuItem.setSelected(true);
        slowMenuItem.setText("Slow");
        mediumMenuItem.setText("Medium");
        fastMenuItem.setText("Fast");
        jMenuFile.add(jMenuFileExit);
        jMenuFile.add(jMenuFileLoad);
        jMenuFile.add(jMenuFileSave);
        jMenuFile.add(jMenuFileExit);
        jMenuHelp.add(jMenuHelpAbout);
        jMenuBar1.add(jMenuFile);
        jMenuBar1.add(runMenu);
        jMenuBar1.add(jMenuHelp);
        this.setJMenuBar(jMenuBar1);
        contentPane.add(statusBar, BorderLayout.SOUTH);
        contentPane.add(jSplitPane1, BorderLayout.CENTER);
        jSplitPane1.add(jPanel1, JSplitPane.TOP);
        jSplitPane1.add(jSplitPane2, JSplitPane.BOTTOM);
        runMenu.add(menuItemCompile);
        runMenu.add(menuItemRun);
        runMenu.add(traceMenuItem);
        runMenu.add(slowMenuItem);
        runMenu.add(mediumMenuItem);
        runMenu.add(fastMenuItem);
        runMenu.add(randomizeMenuItem);
        jSplitPane2.add(jScrollPane1, JSplitPane.TOP);
        jScrollPane1.getViewport().add(editTextComponent, null);
        jSplitPane2.add(jScrollPane2, JSplitPane.BOTTOM);
        jScrollPane2.getViewport().add(messageTextComponent, null);
        menuItemCompile.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                compile();
            }
        });
        menuItemRun.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                run();
            }
        });
        jSplitPane2.setDividerLocation(200);
    }
    //File | Exit action performed
    public void jMenuFileExit_actionPerformed(ActionEvent e) {
        System.exit(0);
    }
    //Help | About action performed
    public void jMenuHelpAbout_actionPerformed(ActionEvent e) {
        MainFrame_AboutBox dlg = new MainFrame_AboutBox(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.setModal(true);
        dlg.show();
    }
    
    //Run | Compile
    //Overridden so we can exit when window is closed
    protected void compile() {
        String program = editTextComponent.getText() ;
        facade.compile( program, stringSink );
    }
    
    //Run | Run
    //Overridden so we can exit when window is closed
    protected void run() {
        int delay ;
        if( slowMenuItem.isSelected() ) delay = 100 ;
        else if( mediumMenuItem.isSelected() ) delay = 2;
        else delay = 0 ;
        facade.run(intSource, stringSink, intSink, traceMenuItem.isSelected(), delay);
    }

    // File | Save
    void saveFile(ActionEvent e) {
        /** @todo Add file filtering. */
        int choice = fileChooser.showSaveDialog(this) ;
        if( choice == JFileChooser.APPROVE_OPTION ) {
            File file = fileChooser.getSelectedFile() ;
            if( file.exists() && ! file.canWrite() ) {
                JOptionPane.showMessageDialog(this,
                            "Can not write file " + file,
                            "Error",
                            JOptionPane.ERROR_MESSAGE ); }
            else {
                try {
                    OutputStream outStream = new FileOutputStream( file ) ;
                    PrintWriter pw = new PrintWriter( outStream ) ;
                    String buf = editTextComponent.getText() ;
                    pw.write( buf );
                    pw.close(); }
                catch( FileNotFoundException ex) {
                    JOptionPane.showMessageDialog(this,
                            "File not Found " + file,
                            "Error",
                            JOptionPane.ERROR_MESSAGE ); } } }
    }

    // File | Load
    void loadFile(ActionEvent e) {
        /** @todo Add file filtering. */
        int choice = fileChooser.showOpenDialog(this) ;
        if( choice == JFileChooser.APPROVE_OPTION ) {
            File file = fileChooser.getSelectedFile() ;
            if( ! file.canRead() ) {
                JOptionPane.showMessageDialog(this,
                            "Can not read file " + file,
                            "Error",
                            JOptionPane.ERROR_MESSAGE ); }
            else {
                try {
                    Reader reader = new FileReader( file ) ;
                    StringBuffer buf = new StringBuffer() ;
                    while( true ) {
                        int ch = reader.read() ;
                        if( ch == -1 ) break ;
                        buf.append( (char)ch ) ; }
                    reader.close();
                    editTextComponent.setText( buf.toString() ); }
                catch( FileNotFoundException ex) {
                    JOptionPane.showMessageDialog(this,
                            "File not Found " + file,
                            "Error",
                            JOptionPane.ERROR_MESSAGE ); } 
                catch( IOException ex ) {
                    JOptionPane.showMessageDialog(this,
                            "Errors reading file.",
                            "Error",
                            JOptionPane.ERROR_MESSAGE ); } } }
    }

    void randomize(ActionEvent e) {
       facade.randomize() ;
    }
}